/*
 * Copyright (c) 1997 Vectaport Inc.
 * Copyright (c) 1996 Vectaport Inc., R.B. Kissh & Associates 
 * Copyright (c) 1994-1995 Vectaport Inc., Cartoactive Systems
 * Copyright (c) 1990, 1991 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

/*
 * MapImportCmd implementation.
 */

#include <MapUnidraw/mapclasses.h>
#include <MapUnidraw/mapcomps.h>
#include <MapUnidraw/mapeditor.h>
#include <MapUnidraw/mapimport.h>
#include <MapUnidraw/mapviewer.h>
#include <MapUnidraw/mapviewstate.h>
#include <OverlayUnidraw/ovfile.h>
#include <OverlayUnidraw/ovpage.h>
#include <OverlayUnidraw/ovviewer.h>
#include <Unidraw/Commands/edit.h>
#include <Unidraw/Commands/dirty.h>
#include <Unidraw/Graphic/graphic.h>
#include <Unidraw/clipboard.h>
#include <Unidraw/editor.h>
#include <Unidraw/viewer.h>
#include <IVGlyph/importchooser.h>
#include <InterViews/transformer.h>
#include <ComTerp/comvalue.h>
#include <Attribute/attribute.h>
#include <Attribute/attrlist.h>
#include <Attribute/attrvalue.h>
#include <Attribute/paramlist.h>
#include <InterViews/cursor.h>
#include <InterViews/window.h>
#include <ctype.h>
#include <iostream.h>

/*****************************************************************************/

ClassId MapImportCmd::GetClassId () { return MAPIMPORT_CMD; }

boolean MapImportCmd::IsA (ClassId id) {
    return MAPIMPORT_CMD == id || OvImportCmd::IsA(id);
}

MapImportCmd::MapImportCmd (Editor* ed, ImportChooser* f) 
: OvImportCmd(ed, f) { 
  _readonly = false;
}

MapImportCmd::MapImportCmd (ControlInfo* c, ImportChooser* f) 
: OvImportCmd(c, f) { 
  _readonly = false;
}


Command* MapImportCmd::Copy () {
    MapImportCmd* copy = new MapImportCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void MapImportCmd::Execute () { 

    OverlayComp* comp = (OverlayComp*) 
      (inptr_ 
       ? Import(*inptr_) 
       : ( path_ 
	   ? Import(path_) 
	   : PostDialog()));

    if (comp != nil) {
	// Set the view to lie over the imported fclass

	AttributeValue* swea;
	AttributeValue* swna;
	if ((swea = comp->FindValue("sw_e")) 
	    && (swna = comp->FindValue("sw_n")))
	  {
	    if (readonly()) {
	      AttributeList* al = comp->GetAttributeList();
	      al->add_attr("readonly", ComValue::trueval());
	    }

	    double x1 = swea->double_val();
	    double y1 = swna->double_val();
	    Transformer* rel = ((OverlayViewer*)GetEditor()->GetViewer())
	      ->GetRel();
	    float tx1, ty1;
	    tx1 = x1; ty1 = y1;
	    // use the mapviewstate mechanism
	    MapViewState* vs = ((MapEditor*)GetEditor())->GetMapViewState();
	    if (!vs->deftrans()) {
	      // set it according to the first import coming in
	      Transformer* tr = new Transformer();
	      tr->translate(-tx1, -ty1);
	      vs->settrans(tr);
	    }
	    if (!vs->implicitproj()) {
	      AttributeList* al = comp->attrlist();
	      Attribute* pa = al ? al->GetAttr("proj") : nil;
	      if (pa) {
		char* pj = (char*)pa->Value()->string_ptr();
		char* el = (char*)al->GetAttr("ellps")->Value()->string_ptr();
		int zn = al->GetAttr("zone")->Value()->int_val();
		if (pj && el) {
		  char pjs[40];
		  char els[40];
		  char zs[40];
		  sprintf(pjs, "proj=%s", pj);
		  sprintf(els, "ellps=%s", el);
		  sprintf(zs, "zone=%d", zn);
		  char* parms[4] = { pjs, els, zs, "no_defs" };
		  PJ* iproj = pj_init(4, parms);
		  vs->implicitproj(iproj);
		  ((MapEditor*)GetEditor())->SetProjectionLocal(iproj);
		}
	      }
	    }
#if 0
	    AttributeList* al = comp->attrlist();
	    Attribute* scaleattr = al ? al->GetAttr("scale") : nil;
	    if (scaleattr) {
	      float sv = scaleattr->Value()->float_val();
	      comp->GetGraphic()->Scale(sv, sv);
	    }
#endif
	    
#if 0
	    // copy some attributes not present to the top
	    OverlayComp* top = (OverlayComp*)GetEditor()->GetComponent();
	    AttributeList* topal = top->GetAttributeList();
	    if (!topal->GetAttr("sw_e"))
	      topal->Append(new Attribute("sw_e", swea));
	    if (!topal->GetAttr("sw_n"))
	      topal->Append(new Attribute("sw_n", swna));

	    AttributeList* al = comp->attrlist();
	    if (al) {
	      if (!topal->GetAttr("proj") && al->GetAttr("proj"))
		topal->Append(new Attribute(*al->GetAttr("proj")));
	      if (!topal->GetAttr("ellps") && al->GetAttr("ellps"))
		topal->Append(new Attribute(*al->GetAttr("ellps")));
	      if (!topal->GetAttr("zone") && al->GetAttr("zone"))
		topal->Append(new Attribute(*al->GetAttr("zone")));
	      if (!topal->GetAttr("dtedlos") && al->GetAttr("dtedlos"))
		topal->Append(new Attribute(*al->GetAttr("dtedlos")));
	      if (!topal->GetAttr("demlos") && al->GetAttr("demlos"))
		topal->Append(new Attribute(*al->GetAttr("demlos")));
	      if (!topal->GetAttr("initmag") && al->GetAttr("initmag"))
		topal->Append(new Attribute(*al->GetAttr("initmag")));
	      if (!topal->GetAttr("initlat") && al->GetAttr("initlat"))
		topal->Append(new Attribute(*al->GetAttr("initlat")));
	      if (!topal->GetAttr("initlon") && al->GetAttr("initlon"))
		topal->Append(new Attribute(*al->GetAttr("initlon")));
	    }
#endif

	    MapViewer* v = (MapViewer*) GetEditor()->GetViewer();
	    v->SetMapPage(comp);

	    PasteCmd* paste_cmd = new PasteCmd(GetEditor(),
					       new Clipboard(comp));
	    paste_cmd->Execute();
	    paste_cmd->Log();
	    
	    /* undo the dirty command */
	    if (readonly()) {
	      DirtyCmd* dirty_cmd = new DirtyCmd(GetEditor());
	      dirty_cmd->reverse() = true;
	      dirty_cmd->Execute();
	    }
	  }
	else {
	  cerr << "could not find :sw_e or :sw_n attribute\n";
	  GetEditor()->GetWindow()->cursor(arrow);
	}
    } else
      cerr << "null map import\n";
}

GraphicComp* MapImportCmd::Import (istream& in) {
    OverlayComp* comp = nil;
    Editor* ed = GetEditor();

    int len = 255;
    char buf[len];

    char ch;
    while (isspace(ch = in.get())); in.putback(ch);
    ParamList::parse_token(in, buf, len);

    OverlayComp* parent_comp = (OverlayComp*)ed->GetComponent();

    if (strcmp(buf, "drawtool") == 0 || strcmp(buf, "ov-idraw") == 0) { 
        comp = new OverlayIdrawComp(in, "ACE", parent_comp);
	if (comp) {
	    OverlayFileComp* ovfile = new OverlayFileComp();
	    ovfile->Append(comp);
	    comp = ovfile;
	}
    } else if (strcmp(buf, "mapfcl") == 0) { 
        comp = new MapFClassComp(in, parent_comp);
    } 
    if (chooser_ && comp)
      comp->SetByPathnameFlag(chooser_->by_pathname());

    return comp;
}

GraphicComp* MapImportCmd::Import (const char* pathname) {
    Editor* ed = GetEditor();
    const char* creator = ReadCreator(pathname);
    if (!creator) return nil;
    OverlayComp* comp = nil;
    OverlayComp* parent_comp = (OverlayComp*)ed->GetComponent();

    if (strcmp(creator, "X11") == 0) {
      comp = (OverlayComp*)XBitmap_Image(pathname);
      
    } else if (strcmp(creator, "TIFF") == 0) {
      comp = (OverlayComp*)TIFF_Image(pathname);
      
    } else if (strcmp(creator, "PBM") == 0) {
      comp = (OverlayComp*)PBM_Image(pathname);
      
    } else if (strcmp(creator, "PGM") == 0) {
      comp = (OverlayComp*)PGM_Image(pathname);
      
    } else if (strcmp(creator, "PPM") == 0) {
      comp = (OverlayComp*)PPM_Image(pathname);
    } else {
	filebuf fbuf;
	fbuf.open(pathname, "r");
	istream in(&fbuf);
	char ch; while (isspace(ch = in.get())); in.putback(ch);
	int len = 255;
	char buf[len];
	ParamList::parse_token(in, buf, len);

	if (strcmp(buf, "drawtool") == 0 || strcmp(buf, "ov-idraw") == 0) { 
	  comp = new OverlayIdrawComp(in, pathname, parent_comp);
	  if (comp) {
	    OverlayFileComp* ovfile = new OverlayFileComp();
	    ovfile->Append(comp);
	    comp = ovfile;
	  }
	} else if (strcmp(buf, "mapserv") == 0) { 
	  comp = new MapIdrawComp(in, pathname, parent_comp);
	  if (comp) {
	    OverlayFileComp* ovfile = new OverlayFileComp();
	    ovfile->Append(comp);
	    comp = ovfile;
	  }
	} else if (strcmp(buf, "mapfcl") == 0) { 
	  comp = new MapFClassComp(in, parent_comp);
	}
	if (chooser_ && comp)
	  comp->SetByPathnameFlag(chooser_->by_pathname());
    }
    if (comp) ((OverlayComp*)comp)->SetPathName(pathname);
    return comp;
}
