/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

/* Class wrapper for vpf feature class object */

#ifndef vpf_fclass_h
#define vpf_fclass_h

#include <Map/mapfclass.h>
#include <Vpf/vpf.h>

class MapFace;
class VpfCoverage;
class VpfColumnHeader;
class VpfFeature;

class VpfFeatureClass : public MapFeatureClass {
public:
    VpfFeatureClass (VpfCoverage*, const char* name,
		     boolean makefeatures =false);
    ~VpfFeatureClass ();

    VpfCoverage* coverage() const { return (VpfCoverage*)_coverage; }
    virtual void coverage(VpfCoverage* cov);

    virtual const char* description() const;

    const char* featuretable () const;
    const char* primitivetable () const;

    /* table methods */
    int is_primitive () const;
    int is_simple_feature () const;
    int is_complex_feature () const;
    int is_feature () const;
    int is_join () const;
    int primitive_class () const;
    int feature_class_type () const;

    static int is_primitive (const char*);
    static int is_simple_feature (const char*);
    static int is_complex_feature (const char*);
    static int is_feature (const char*);
    static int is_join (const char*);
    static int primitive_class (const char*);
    static int feature_class_type (const char*);

    VpfFeature* feature(int) const;
    void delete_features();

    int ncolumns() const { return _ncolumns; }
    VpfColumnHeader* column(int i) const { return _columns[i]; }
    long int column_pos(char* colname) const;

    void print() const;
    void dump_table() const;
    void dump_prim_table() const ;
    void dump_ring_table() const;
    void dump_dict() const;

  void tilename(char* tn);
  char* tilename() { return _tilename; }
protected:
  char* _tilename;
    boolean _makefeatures;
    char* _desc;
    char* _tabl;
    char* _tdesc;
    char* _ptable;
    int _isprim, _issimpf, _iscompf, _isfeat, _isjoin, _primclass, _fctype;
    void add_polygon_edge(edge_rec_type *edge_rec,
			  float*& x, float*& y,
			  int* n, int* size) const;
    long int next_polygon_edge( edge_rec_type *edge_rec, long int *prevnode,
			       long int face_id ) const;
    MapFace* outline_polygon_loop(row_type, long int face_id,
				     long int start_edge,
				     vpf_table_type edgetable, int) const;
    VpfFeature* outline_face( row_type, long int feat_id, long int face_id,
			     vpf_table_type facetable,
			     vpf_table_type ringtable,
			     vpf_table_type edgetable,
			     int ftype,
			     int inner) const;
    int _ncolumns;
    VpfColumnHeader** _columns;
    void make_columns();

    virtual MapFeature** make_features(int &nfeats);
};

class VpfColumnHeader {
public:
    VpfColumnHeader(header_type header);
    VpfColumnHeader(header_cell header);
    ~VpfColumnHeader();
    VpfColumnHeader* copy();
    const char* name() const { return _hc.name; }
    const char* description() const { return _hc.description; }
    char keytype() const { return _hc.keytype; }
    const char* value_description_table() const { return _hc.vdt; }
    char type() const { return _hc.type; }
    long int count() const { return _hc.count; }
    null_field nullval() const { return _hc.nullval; }
    const char* narrative() const { return _hc.narrative; }
    void print() const;
protected:
    header_cell _hc;
};

#endif /* defined(vpf_fclass_h) */
